import { booleanConverter, Button, Color, CssProperty, Property, Style } from '@nativescript/core';
import { BoxType } from './common';
const checkBoxBackgroundColorProperty = new CssProperty({
    name: 'checkBoxBackgroundColor',
    cssName: 'checkbox-background-color',
    valueConverter: v => {
        return String(v);
    }
});
const onCheckColorProperty = new CssProperty({
    name: 'onCheckColor',
    cssName: 'on-check-color',
    defaultValue: '#ffffff',
    valueConverter: v => {
        return String(v);
    }
});
const tintColorProperty = new CssProperty({
    name: 'tintColor',
    cssName: 'tint-color',
    valueConverter: v => {
        return String(v);
    }
});
const onTintColorProperty = new CssProperty({
    name: 'onTintColor',
    cssName: 'on-tint-color',
    valueConverter: v => {
        return String(v);
    }
});
const fillColorProperty = new CssProperty({
    name: 'fillColor',
    cssName: 'fill-color',
    valueConverter: v => {
        return String(v);
    }
});
const checkedProperty = new Property({
    name: 'checked',
    defaultValue: false,
    valueConverter: booleanConverter,
    valueChanged: onCheckedPropertyChanged
});
const boxTypeProperty = new Property({
    name: 'boxType',
    valueConverter: v => {
        return BoxType[v] === BoxType.circle
            ? 0
            : 1;
    }
});
export class CheckBox extends Button {
    constructor() {
        super(...arguments);
        this._lineWidth = 1;
    }
    createNativeView() {
        return BEMCheckBox.alloc().initWithFrame(CGRectMake(0, 0, 80, 80));
    }
    set fillColor(color) {
        this._fillColor = color;
        this.nativeViewProtected.onFillColor = new Color(color).ios;
    }
    set onFillColor(color) {
        this._onFillColor = color;
        this.nativeViewProtected.onFillColor = new Color(color).ios;
    }
    set tintColor(color) {
        this._tintColor = color;
        this.nativeViewProtected.tintColor = new Color(color).ios;
    }
    set onTintColor(color) {
        this._onTintColor = color;
        this.nativeViewProtected.onTintColor = new Color(color).ios;
    }
    set checkBoxBackgroundColor(color) {
        this._checkBoxBackgroundColor = color;
        this.nativeViewProtected.offFillColor = new Color(color).ios;
    }
    set onCheckColor(color) {
        this._onCheckColor = color;
        this.nativeViewProtected.onCheckColor = new Color(color).ios;
    }
    [boxTypeProperty.setNative](value) {
        if (this.nativeViewProtected) {
            this.nativeViewProtected.boxType = value;
        }
    }
    [checkedProperty.getDefault]() {
        return false;
    }
    [checkedProperty.setNative](value) {
        this.nativeViewProtected.setOnAnimated(value, true);
    }
    set checkedAnimated(value) {
        this.nativeViewProtected.setOnAnimated(value, true);
    }
    set lineWidth(value) {
        this.nativeViewProtected.lineWidth = value;
        this._lineWidth = value;
    }
    set hideBox(value) {
        this.nativeViewProtected.hideBox = value;
        this._hideBox = value;
    }
    set animationDuration(value) {
        this.nativeViewProtected.animationDuration = value;
        this._animationDuration = value;
    }
    set onAnimationType(value) {
        if (this.nativeViewProtected)
            this.nativeViewProtected.onAnimationType = this._getAnimationType(value);
        else
            this._onAnimationType = value;
    }
    set offAnimationType(value) {
        this.nativeViewProtected.offAnimationType = this._getAnimationType(value);
        this._offAnimationType = value;
    }
    get nativeiOSCheckBox() {
        return this.nativeViewProtected;
    }
    reload(value) {
        this.nativeViewProtected.reload();
    }
    initNativeView() {
        this.addEventListener('tap', args => {
            this.tapTimeout = setTimeout(() => {
                this.tapTimeout = null;
                const checkbox = args.object;
                checkbox.checked = !checkbox.checked;
            }, 10);
        });
        this._onAnimationType = 2;
        this._offAnimationType = 2;
        this._delegate = BEMCheckBoxDelegateImpl.initWithOwner(new WeakRef(this));
        let fontSize;
        if (!this.style.fontSize) {
            fontSize = 15;
        }
        else {
            fontSize = this.style.fontSize;
        }
        this.nativeViewProtected.delegate = this._delegate;
        this.nativeViewProtected.frame = CGRectMake(0, 0, fontSize, fontSize);
        this.nativeViewProtected.center = CGPointMake(this.nativeViewProtected.center.x, fontSize / 2 + 3);
        this.style.paddingLeft = fontSize + (fontSize > 20 ? 10 : 5);
        this.style.textAlignment = 'left';
        if (this._onCheckColor) {
            this.nativeViewProtected.onCheckColor = new Color(this._onCheckColor).ios;
        }
        if (this._onFillColor) {
            this.nativeViewProtected.onFillColor = new Color(this._onFillColor).ios;
        }
        if (this._onTintColor) {
            this.nativeViewProtected.onTintColor = new Color(this._onTintColor).ios;
        }
        if (this._fillColor) {
            this.nativeViewProtected.onFillColor = new Color(this._fillColor).ios;
        }
        if (this._tintColor) {
            this.nativeViewProtected.tintColor = new Color(this._tintColor).ios;
        }
        this.nativeView.addSubview(this.nativeViewProtected);
        if (typeof this._lineWidth !== 'undefined') {
            this.lineWidth = this._lineWidth;
        }
        if (typeof this._hideBox !== 'undefined') {
            this.hideBox = this._hideBox;
        }
        this.boxType = this.boxType === 0 ? 0 : 1;
        if (typeof this._animationDuration !== 'undefined') {
            this.animationDuration = this._animationDuration;
        }
        if (typeof this._onAnimationType !== 'undefined') {
            this.onAnimationType = this._onAnimationType;
        }
        if (typeof this._offAnimationType !== 'undefined') {
            this.offAnimationType = this._offAnimationType;
        }
    }
    disposeNativeView() {
        this.nativeViewProtected.delegate = null;
        this.removeEventListener('tap');
    }
    toggle() {
        this.checked = !this.checked;
    }
    _getAnimationType(value) {
        switch (value) {
            case 1:
                return 0;
            case 2:
                return 1;
            case 3:
                return 2;
            case 4:
                return 3;
            case 5:
                return 0;
            case 6:
                return 5;
            default:
                return 0;
        }
    }
    _onCheckedPropertyChanged(checkbox, oldValue, newValue) {
        if (!this.nativeView) {
            return;
        }
        checkedProperty.nativeValueChange(this, newValue);
    }
}
var BEMCheckBoxDelegateImpl = /** @class */ (function (_super) {
    __extends(BEMCheckBoxDelegateImpl, _super);
    function BEMCheckBoxDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /* static ObjCExposedMethods = {
       "didTapCheckBox": { returns: interop.types.void, params: [NSObject] }
       };*/
    BEMCheckBoxDelegateImpl.initWithOwner = function (owner) {
        var delegate = BEMCheckBoxDelegateImpl.new();
        delegate._owner = owner;
        return delegate;
    };
    BEMCheckBoxDelegateImpl.prototype.animationDidStopForCheckBox = function (checkBox) {
        // TODO: Maybe trigger event later?
    };
    BEMCheckBoxDelegateImpl.prototype.didTapCheckBox = function (checkBox) {
        var owner = this._owner.get();
        if (owner) {
            if (owner.tapTimeout) {
                clearTimeout(owner.tapTimeout);
                owner.tapTimeout = null;
            }
            checkedProperty.nativeValueChange(owner, checkBox.on);
        }
    };
    BEMCheckBoxDelegateImpl.ObjCProtocols = [BEMCheckBoxDelegate];
    return BEMCheckBoxDelegateImpl;
}(NSObject));
function onCheckedPropertyChanged(checkbox, oldValue, newValue) {
    checkbox._onCheckedPropertyChanged(checkbox, oldValue, newValue);
}
boxTypeProperty.register(CheckBox);
checkedProperty.register(CheckBox);
fillColorProperty.register(Style);
onTintColorProperty.register(Style);
onCheckColorProperty.register(Style);
checkBoxBackgroundColorProperty.register(Style);
tintColorProperty.register(Style);
//# sourceMappingURL=checkbox.ios.js.map